@@ -29,18 +29,12 @@
 #include "gnome-vfs-mime.h"
 #include "gnome-vfs-result.h"
 #include "gnome-vfs-utils.h"
-#ifdef HAVE_GCONF
-#include <gconf/gconf.h>
-#endif
+#include <bonobo-activation/bonobo-activation-activate.h>
+#include <gconf/gconf-client.h>
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_GCONF
-static GConfEngine *gconf_engine = NULL;
-#endif
-
-static char *         get_user_level                          (void);
-static GList *        OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list);
+static GList *        Bonobo_ServerInfoList_to_ServerInfo_g_list (Bonobo_ServerInfoList *info_list);
 static GList *        copy_str_list                           (GList *string_list);
 static GList *        comma_separated_str_to_str_list         (const char         *str);
 static GList *        str_list_difference                     (GList              *a,
@@ -52,6 +46,8 @@ static GnomeVFSResult gnome_vfs_mime_edit_user_file           (const char
 							       const char         *key,
 							       const char         *value);
 static gboolean       application_known_to_be_nonexistent     (const char         *application_id);
+static const char    *gnome_vfs_mime_maybe_get_user_level_value (const char         *mime_type,
+								 const char         *key);
 
 /**
  * gnome_vfs_mime_get_description:
@@ -101,9 +97,9 @@ gnome_vfs_mime_get_default_action_type (const char *mime_type)
 
 	action_type_string = gnome_vfs_mime_get_value (mime_type, "default_action_type");
 
-	if (action_type_string != NULL && strcasecmp (action_type_string, "application") == 0) {
+	if (action_type_string != NULL && g_ascii_strcasecmp (action_type_string, "application") == 0) {
 		return GNOME_VFS_MIME_ACTION_TYPE_APPLICATION;
-	} else if (action_type_string != NULL && strcasecmp (action_type_string, "component") == 0) {
+	} else if (action_type_string != NULL && g_ascii_strcasecmp (action_type_string, "component") == 0) {
 		return GNOME_VFS_MIME_ACTION_TYPE_COMPONENT;
 	} else {
 		return GNOME_VFS_MIME_ACTION_TYPE_NONE;
@@ -294,15 +290,15 @@ gnome_vfs_mime_set_can_be_executable (const char *mime_type, gboolean new_value)
  * Query the MIME database for the default Bonobo component to be activated to 
  * view files of MIME type @mime_type.
  * 
- * Return value: An OAF_ServerInfo * representing the OAF server to be activated
+ * Return value: An Bonobo_ServerInfo * representing the OAF server to be activated
  * to get a reference to the proper component.
  **/
-OAF_ServerInfo *
+Bonobo_ServerInfo *
 gnome_vfs_mime_get_default_component (const char *mime_type)
 {
 	const char *default_component_iid;
-	OAF_ServerInfoList *info_list;
-	OAF_ServerInfo *default_component;
+	Bonobo_ServerInfoList *info_list;
+	Bonobo_ServerInfo *default_component;
 	CORBA_Environment ev;
 	char *supertype;
 	char *query;
@@ -348,10 +344,10 @@ gnome_vfs_mime_get_default_component (const char *mime_type)
  			prev = sort[1];
 			
 			if (p->next != NULL) {
-				sort[1] = g_strconcat (prev, ((OAF_ServerInfo *) (p->data))->iid, 
+				sort[1] = g_strconcat (prev, ((Bonobo_ServerInfo *) (p->data))->iid, 
 								    "','", NULL);
 			} else {
-				sort[1] = g_strconcat (prev, ((OAF_ServerInfo *) (p->data))->iid, 
+				sort[1] = g_strconcat (prev, ((Bonobo_ServerInfo *) (p->data))->iid, 
 								    "'])", NULL);
 			}
 			g_free (prev);
@@ -372,12 +368,12 @@ gnome_vfs_mime_get_default_component (const char *mime_type)
 	sort[4] = g_strdup ("name");
 	sort[5] = NULL;
 
-	info_list = oaf_query (query, sort, &ev);
+	info_list = bonobo_activation_query (query, sort, &ev);
 	
 	default_component = NULL;
 	if (ev._major == CORBA_NO_EXCEPTION) {
 		if (info_list != NULL && info_list->_length > 0) {
-			default_component = OAF_ServerInfo_duplicate (&info_list->_buffer[0]);
+			default_component = Bonobo_ServerInfo_duplicate (&info_list->_buffer[0]);
 		}
 		CORBA_free (info_list);
 	}
@@ -434,35 +430,13 @@ gnome_vfs_mime_str_list_apply_delta (GList *list_to_process,
 	return processed_list;
 }
 
-static const char *
-gnome_vfs_mime_get_value_for_user_level (const char *mime_type, 
-					 const char *key_prefix)
-{
-	char *user_level;
-	char *full_key;
-	const char *value;
-
-	/* Base list depends on user level. */
-	user_level = get_user_level ();
-	full_key = g_strconcat (key_prefix,
-				"_for_",
-				user_level,
-				"_user_level",
-				NULL);
-	g_free (user_level);
-	value = gnome_vfs_mime_get_value (mime_type, full_key);
-	g_free (full_key);
-
-	return value;
-}
-
-
-static GList *gnome_vfs_mime_do_short_list_processing (GList *short_list, 
-						       GList *additions,
-						       GList *removals, 
-						       GList *supertype_short_list, 
-						       GList *supertype_additions, 
-						       GList *supertype_removals)
+static GList *
+gnome_vfs_mime_do_short_list_processing (GList *short_list, 
+					 GList *additions,
+					 GList *removals, 
+					 GList *supertype_short_list, 
+					 GList *supertype_additions, 
+					 GList *supertype_removals)
 {
 	GList *processed_supertype_list;
 	GList *merged_system_and_supertype;
@@ -499,7 +473,7 @@ sort_application_list (gconstpointer a, gconstpointer b)
 	application1 = (GnomeVFSMimeApplication *) a;
 	application2 = (GnomeVFSMimeApplication *) b;
 
-	return g_strcasecmp (application1->name, application2->name);
+	return g_ascii_strcasecmp (application1->name, application2->name);
 }
 
 /**
@@ -534,7 +508,7 @@ gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 	}
 
 
-	system_short_list = comma_separated_str_to_str_list (gnome_vfs_mime_get_value_for_user_level 
+	system_short_list = comma_separated_str_to_str_list (gnome_vfs_mime_maybe_get_user_level_value
 							     (mime_type, 
 							      "short_list_application_ids"));
 	system_short_list = prune_ids_for_nonexistent_applications
@@ -556,7 +530,7 @@ gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 
 	if (!gnome_vfs_mime_type_is_supertype (mime_type) && system_short_list == NULL) {
 		supertype_short_list = comma_separated_str_to_str_list 
-			(gnome_vfs_mime_get_value_for_user_level 
+			(gnome_vfs_mime_maybe_get_user_level_value
 			 (supertype, 
 			  "short_list_application_ids"));
 		supertype_short_list = prune_ids_for_nonexistent_applications
@@ -647,12 +621,12 @@ join_str_list (const char *separator, GList *list)
  * gnome_vfs_mime_get_short_list_components:
  * @mime_type: A const char * containing a mime type, e.g. "image/png"
  * 
- * Return an unsorted sorted list of OAF_ServerInfo *
+ * Return an unsorted sorted list of Bonobo_ServerInfo *
  * data structures for the requested mime type.	The short list contains
  * "select" components recommended for handling this MIME type, appropriate for
  * display to the user.
  * 
- * Return value: A GList * where the elements are OAF_ServerInfo *
+ * Return value: A GList * where the elements are Bonobo_ServerInfo *
  * representing various components to display in the short list for @mime_type.
  **/ 
 GList *
@@ -670,7 +644,7 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 	char *sort[2];
 	char *iids_delimited;
 	CORBA_Environment ev;
-	OAF_ServerInfoList *info_list;
+	Bonobo_ServerInfoList *info_list;
 	GList *preferred_components;
 
 	if (mime_type == NULL) {
@@ -679,7 +653,7 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 
 
 	/* get short list IIDs for that user level */
-	system_short_list = comma_separated_str_to_str_list (gnome_vfs_mime_get_value_for_user_level 
+	system_short_list = comma_separated_str_to_str_list (gnome_vfs_mime_get_value
 							     (mime_type, 
 							      "short_list_component_iids"));
 
@@ -698,7 +672,7 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 	
 	if (strcmp (supertype, mime_type) != 0) {
 		supertype_short_list = comma_separated_str_to_str_list 
-			(gnome_vfs_mime_get_value_for_user_level 
+			(gnome_vfs_mime_get_value
 			 (supertype, 
 			  "short_list_component_iids"));
 
@@ -746,10 +720,10 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 		sort[0] = g_strconcat ("prefer_by_list_order(iid, ['", iids_delimited, "'])", NULL);
 		sort[1] = NULL;
 		
-		info_list = oaf_query (query, sort, &ev);
+		info_list = bonobo_activation_query (query, sort, &ev);
 		
 		if (ev._major == CORBA_NO_EXCEPTION) {
-			preferred_components = OAF_ServerInfoList_to_ServerInfo_g_list (info_list);
+			preferred_components = Bonobo_ServerInfoList_to_ServerInfo_g_list (info_list);
 			CORBA_free (info_list);
 		}
 
@@ -830,17 +804,17 @@ gnome_vfs_mime_get_all_applications (const char *mime_type)
  * gnome_vfs_mime_get_all_components:
  * @mime_type: A const char * containing a mime type, e.g. "image/png"
  * 
- * Return an alphabetically sorted list of OAF_ServerInfo
+ * Return an alphabetically sorted list of Bonobo_ServerInfo
  * data structures representing all Bonobo components registered
  * to handle files of MIME type @mime_type (and supertypes).
  * 
- * Return value: A GList * where the elements are OAF_ServerInfo *
+ * Return value: A GList * where the elements are Bonobo_ServerInfo *
  * representing components that can handle MIME type @mime_type.
  **/ 
 GList *
 gnome_vfs_mime_get_all_components (const char *mime_type)
 {
-	OAF_ServerInfoList *info_list;
+	Bonobo_ServerInfoList *info_list;
 	GList *components_list;
 	CORBA_Environment ev;
 	char *supertype;
@@ -870,10 +844,10 @@ gnome_vfs_mime_get_all_components (const char *mime_type)
 	sort[0] = g_strdup ("name");
 	sort[1] = NULL;
 
-	info_list = oaf_query (query, sort, &ev);
+	info_list = bonobo_activation_query (query, sort, &ev);
 	
 	if (ev._major == CORBA_NO_EXCEPTION) {
-		components_list = OAF_ServerInfoList_to_ServerInfo_g_list (info_list);
+		components_list = Bonobo_ServerInfoList_to_ServerInfo_g_list (info_list);
 		CORBA_free (info_list);
 	} else {
 		components_list = NULL;
@@ -907,7 +881,7 @@ gnome_vfs_mime_edit_user_file_full (const char *mime_type, GList *keys, GList *v
 		if (value == NULL) {
 			value = "";
 		}
-		gnome_vfs_mime_set_value (mime_type, g_strdup (key), g_strdup (value));
+		gnome_vfs_mime_set_value (mime_type, key, value);
 	}
 	gnome_vfs_mime_thaw ();
 
@@ -1067,7 +1041,6 @@ GnomeVFSResult
 gnome_vfs_mime_set_short_list_applications (const char *mime_type,
 					    GList *application_ids)
 {
-	char *user_level, *id_list_key;
 	char *addition_string, *removal_string;
 	GList *short_list_id_list;
 	GList *short_list_addition_list;
@@ -1075,16 +1048,8 @@ gnome_vfs_mime_set_short_list_applications (const char *mime_type,
 	GnomeVFSResult result;
 
 	/* Get base list. */
-	/* Base list depends on user level. */
-	user_level = get_user_level ();
-	id_list_key = g_strconcat ("short_list_application_ids_for_",
-				   user_level,
-				   "_user_level",
-				   NULL);
-	g_free (user_level);
 	short_list_id_list = comma_separated_str_to_str_list
-		(gnome_vfs_mime_get_value (mime_type, id_list_key));
-	g_free (id_list_key);
+		(gnome_vfs_mime_maybe_get_user_level_value (mime_type, "short_list_application_ids"));
 
 	/* Compute delta. */
 	short_list_addition_list = str_list_difference (application_ids, short_list_id_list);
@@ -1123,24 +1088,14 @@ GnomeVFSResult
 gnome_vfs_mime_set_short_list_components (const char *mime_type,
 					  GList *component_iids)
 {
-	char *user_level, *id_list_key;
 	char *addition_string, *removal_string;
 	GList *short_list_id_list;
 	GList *short_list_addition_list;
 	GList *short_list_removal_list;
 	GnomeVFSResult result;
 
-	/* Get base list. */
-	/* Base list depends on user level. */
-	user_level = get_user_level ();
-	id_list_key = g_strconcat ("short_list_component_iids_for_",
-				   user_level,
-				   "_user_level",
-				   NULL);
-	g_free (user_level);
 	short_list_id_list = comma_separated_str_to_str_list
-		(gnome_vfs_mime_get_value (mime_type, id_list_key));
-	g_free (id_list_key);
+		(gnome_vfs_mime_get_value (mime_type, "short_list_component_iids"));
 
 	/* Compute delta. */
 	short_list_addition_list = str_list_difference (component_iids, short_list_id_list);
@@ -1179,13 +1134,11 @@ gnome_vfs_mime_id_matches_application (const char *id, GnomeVFSMimeApplication *
 	return gnome_vfs_mime_application_has_id (application, id);
 }
 
-#ifdef USING_OAF
 static gint
-gnome_vfs_mime_id_matches_component (const char *iid, OAF_ServerInfo *component)
+gnome_vfs_mime_id_matches_component (const char *iid, Bonobo_ServerInfo *component)
 {
 	return strcmp (component->iid, iid);
 }
-#endif
 
 static gint 
 gnome_vfs_mime_application_matches_id (GnomeVFSMimeApplication *application, const char *id)
@@ -1193,13 +1146,11 @@ gnome_vfs_mime_application_matches_id (GnomeVFSMimeApplication *application, con
 	return gnome_vfs_mime_id_matches_application (id, application);
 }
 
-#ifdef USING_OAF
 static gint 
-gnome_vfs_mime_component_matches_id (OAF_ServerInfo *component, const char *iid)
+gnome_vfs_mime_component_matches_id (Bonobo_ServerInfo *component, const char *iid)
 {
 	return gnome_vfs_mime_id_matches_component (iid, component);
 }
-#endif
 
 /**
  * gnome_vfs_mime_id_in_application_list:
@@ -1222,10 +1173,10 @@ gnome_vfs_mime_id_in_application_list (const char *id, GList *applications)
 /**
  * gnome_vfs_mime_id_in_component_list:
  * @iid: A component iid.
- * @components: A GList * whose nodes are OAF_ServerInfos, such as the
+ * @components: A GList * whose nodes are Bonobo_ServerInfos, such as the
  * result of gnome_vfs_mime_get_short_list_components().
  * 
- * Check whether a component iid is in a list of OAF_ServerInfos.
+ * Check whether a component iid is in a list of Bonobo_ServerInfos.
  * 
  * Return value: TRUE if a component whose iid matches @iid is in @components.
  */
@@ -1267,12 +1218,12 @@ gnome_vfs_mime_id_list_from_application_list (GList *applications)
 
 /**
  * gnome_vfs_mime_id_list_from_component_list:
- * @components: A GList * whose nodes are OAF_ServerInfos, such as the
+ * @components: A GList * whose nodes are Bonobo_ServerInfos, such as the
  * result of gnome_vfs_mime_get_short_list_components().
  * 
- * Create a list of component iids from a list of OAF_ServerInfos.
+ * Create a list of component iids from a list of Bonobo_ServerInfos.
  * 
- * Return value: A new list where each OAF_ServerInfo in the original
+ * Return value: A new list where each Bonobo_ServerInfo in the original
  * list is replaced by a char * with the component's iid. The original list is
  * not modified.
  */
@@ -1284,7 +1235,7 @@ gnome_vfs_mime_id_list_from_component_list (GList *components)
 
 	for (node = components; node != NULL; node = node->next) {
 		list = g_list_prepend 
-			(list, g_strdup (((OAF_ServerInfo *)node->data)->iid));
+			(list, g_strdup (((Bonobo_ServerInfo *)node->data)->iid));
 	}
 	return g_list_reverse (list);
 }
@@ -1437,13 +1388,13 @@ gnome_vfs_mime_add_component_to_short_list (const char *mime_type,
 
 /**
  * gnome_vfs_mime_remove_component_from_list:
- * @components: A GList * whose nodes are OAF_ServerInfos, such as the
+ * @components: A GList * whose nodes are Bonobo_ServerInfos, such as the
  * result of gnome_vfs_mime_get_short_list_components().
  * @iid: The iid of a component to remove from @components.
  * @did_remove: If non-NULL, this is filled in with TRUE if the component
  * was found in the list, FALSE otherwise.
  * 
- * Remove a component specified by iid from a list of OAF_ServerInfos.
+ * Remove a component specified by iid from a list of Bonobo_ServerInfos.
  * 
  * Return value: The modified list. If the component is not found, the list will 
  * be unchanged.
@@ -1786,9 +1737,9 @@ gnome_vfs_mime_application_list_free (GList *list)
 
 /**
  * gnome_vfs_mime_component_list_free:
- * @list: a GList of OAF_ServerInfo * to be freed
+ * @list: a GList of Bonobo_ServerInfo * to be freed
  * 
- * Frees lists of OAF_ServerInfo * (as returned from functions such
+ * Frees lists of Bonobo_ServerInfo * (as returned from functions such
  * as @gnome_vfs_get_all_components)
  * 
  **/
@@ -1851,7 +1802,7 @@ prune_ids_for_nonexistent_applications (GList *list)
 }
 
 static GList *
-OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list)
+Bonobo_ServerInfoList_to_ServerInfo_g_list (Bonobo_ServerInfoList *info_list)
 {
 	GList *retval;
 	int i;
@@ -1859,7 +1810,7 @@ OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list)
 	retval = NULL;
 	if (info_list != NULL && info_list->_length > 0) {
 		for (i = 0; i < info_list->_length; i++) {
-			retval = g_list_prepend (retval, OAF_ServerInfo_duplicate (&info_list->_buffer[i]));
+			retval = g_list_prepend (retval, Bonobo_ServerInfo_duplicate (&info_list->_buffer[i]));
 		}
 		retval = g_list_reverse (retval);
 	}
@@ -1867,56 +1818,6 @@ OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list)
 	return retval;
 }
 
-#ifdef HAVE_GCONF
-static void
-unref_gconf_engine (void)
-{
-	gconf_engine_unref (gconf_engine);
-}
-#endif /* HAVE_GCONF */
-
-/* Returns the Nautilus user level, a string.
- * This does beg the question: Why does gnome-vfs have the Nautilus
- * user level coded into it? Eventually we might want to call this the
- * GNOME user level or something if we can figure out a clear concept
- * that works across GNOME.
- */
-static char *
-get_user_level (void)
-{
-	char *user_level = NULL;
-
-#ifdef HAVE_GCONF
-	/* Create the gconf engine once. */
-	if (gconf_engine == NULL) {
-		/* This sequence is needed in case no one has initialized GConf. */
-		if (!gconf_is_initialized ()) {
-			char *fake_argv[] = { "gnome-vfs", NULL };
-			gconf_init (1, fake_argv, NULL);
-		}
-
-		gconf_engine = gconf_engine_get_default ();
-		g_atexit (unref_gconf_engine);
-	}
-
-	user_level = gconf_engine_get_string (gconf_engine, "/apps/nautilus/user_level", NULL);
-#endif
-
-	if (user_level == NULL) {
-		user_level = g_strdup ("novice");
-	}
-
-	/* If value is invalid, assume "novice". */
-	if (strcmp (user_level, "novice") != 0 &&
-	    strcmp (user_level, "intermediate") != 0 &&
-	    strcmp (user_level, "advanced") != 0) {
-		g_free (user_level);
-		user_level = g_strdup ("novice");
-	}
-
-	return user_level;
-}
-
 static char **
 strsplit_handle_null (const char *str, const char *delim, int max)
 {
@@ -2011,3 +1912,27 @@ copy_str_list (GList *string_list)
 				       }
 	return g_list_reverse (copy);
 }
+
+static const char *
+gnome_vfs_mime_maybe_get_user_level_value (const char *mime_type, const char *key)
+{
+	const char *return_value;
+	char *new_key;
+	
+	/* This function checks for a key and falls back
+	 * to the "advanced" user level preference if the key
+	 * doesn't exist.
+	 */
+
+	return_value = gnome_vfs_mime_get_value (mime_type, key);
+
+	if (return_value != NULL) {
+		return return_value;
+	}
+
+	new_key = g_strconcat (key, "_for_advanced_user_level", NULL);
+	return_value = gnome_vfs_mime_get_value (mime_type, new_key);
+	g_free (new_key);
+	
+	return return_value;
+}
