@@ -34,7 +34,7 @@
 #include <glib/gstrfuncs.h>
 #include <glib/gthread.h>
 #include <glib/gutils.h>
-
+#include <glib/gunicode.h>
 static gboolean
 is_octal_digit (char ch)
 {
@@ -253,7 +253,7 @@ match_pattern (const char *scanner, const char **resulting_scanner, const char *
 }
 
 GnomeMagicEntry *
-gnome_vfs_mime_magic_parse (const gchar *filename, gint *nents)
+_gnome_vfs_mime_magic_parse (const gchar *filename, gint *nents)
 {
 	GArray *array;
 	GnomeMagicEntry newent, *retval;
@@ -521,20 +521,27 @@ gnome_vfs_mime_try_one_magic_pattern (GnomeVFSMimeSniffBuffer *sniff_buffer,
 	int offset;
 
 	if (sniff_buffer->read_whole_file &&
-	    sniff_buffer->buffer_length < magic_entry->range_start + magic_entry->pattern_length) {
+	    sniff_buffer->buffer_length < magic_entry->range_end + magic_entry->pattern_length) {
 		/* There's no place this pattern could actually match */
 		return FALSE;
 	}
+
 	for (offset = magic_entry->range_start; offset <= magic_entry->range_end; offset++) {
 		/* this check is done only as an optimization
-		 * gnome_vfs_mime_sniff_buffer_get already implements the laziness.
+		 * _gnome_vfs_mime_sniff_buffer_get already implements the laziness.
 		 * This gets called a million times though and every bit performance
 		 * is valuable. This way we avoid making the call.
 		 */
-		if (sniff_buffer->buffer_length < offset + magic_entry->pattern_length &&
-		    !sniff_buffer->read_whole_file) {
-			if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 
-							     offset + magic_entry->pattern_length) != GNOME_VFS_OK) {
+
+		if (sniff_buffer->buffer_length < offset + magic_entry->pattern_length) {
+
+			if (!sniff_buffer->read_whole_file) {
+				if (_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 
+								     offset + magic_entry->pattern_length) != GNOME_VFS_OK) {
+					return FALSE;
+				}
+			} else {
+				/* We have the entire file and the pattern won't fit. Return FALSE */
 				return FALSE;
 			}
 		}
@@ -557,7 +564,7 @@ gnome_vfs_mime_get_magic_table (void)
 	G_LOCK (mime_magic_table_mutex);
 
 	if (mime_magic_table == NULL) {
-		mime_magic_table = gnome_vfs_mime_magic_parse
+		mime_magic_table = _gnome_vfs_mime_magic_parse
 			(SYSCONFDIR "/gnome-vfs-mime-magic" , NULL);
 	}
 
@@ -567,7 +574,7 @@ gnome_vfs_mime_get_magic_table (void)
 }
 
 const char *
-gnome_vfs_mime_get_type_from_magic_table (GnomeVFSMimeSniffBuffer *buffer)
+_gnome_vfs_mime_get_type_from_magic_table (GnomeVFSMimeSniffBuffer *buffer)
 {
 	GnomeMagicEntry *magic_table;
 	
@@ -590,7 +597,7 @@ gnome_vfs_mime_test_get_magic_table (const char *table_path)
 {
 	G_LOCK (mime_magic_table_mutex);
   	if (mime_magic_table == NULL) {
-		mime_magic_table = gnome_vfs_mime_magic_parse (table_path, NULL);
+		mime_magic_table = _gnome_vfs_mime_magic_parse (table_path, NULL);
   	}
 	G_UNLOCK (mime_magic_table_mutex);
 
@@ -688,7 +695,7 @@ gnome_vfs_mime_dump_magic_table (void)
 }
 
 void
-gnome_vfs_mime_clear_magic_table (void)
+_gnome_vfs_mime_clear_magic_table (void)
 {
 	G_LOCK (mime_magic_table_mutex);
   	g_free (mime_magic_table);
@@ -708,65 +715,46 @@ gnome_vfs_mime_clear_magic_table (void)
 const char *
 gnome_vfs_get_mime_type_for_buffer (GnomeVFSMimeSniffBuffer *buffer)
 {
-	return gnome_vfs_get_mime_type_internal (buffer, NULL);
+	return _gnome_vfs_get_mime_type_internal (buffer, NULL);
 }
 
 enum {
 	GNOME_VFS_TEXT_SNIFF_LENGTH = 256
 };
 
+/**
+ * _gnome_vfs_sniff_buffer_looks_like_text:
+ * @sniff_buffer: buffer to examine
+ *
+ * Return value: returns %TRUE if the contents of @sniff_buffer appear to
+ * be text.
+ **/
 gboolean
-gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
+_gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
 {
-	int index;
-	guchar ch;
+	gchar *end;
 	
-	gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_TEXT_SNIFF_LENGTH);
+	_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_TEXT_SNIFF_LENGTH);
 
 	if (sniff_buffer->buffer_length == 0) {
 		return FALSE;
 	}
 	
-	for (index = 0; index < sniff_buffer->buffer_length - 3; index++) {
-		ch = sniff_buffer->buffer[index];
-		if (!g_ascii_isprint (ch) && !g_ascii_isspace (ch)) {
-			/* check if we are dealing with UTF-8 text
-			 * 
-			 *	 bytes | bits | representation
-			 *	     1 |    7 | 0vvvvvvv
-			 *	     2 |   11 | 110vvvvv 10vvvvvv
-			 *	     3 |   16 | 1110vvvv 10vvvvvv 10vvvvvv
-			 *	     4 |   21 | 11110vvv 10vvvvvv 10vvvvvv 10vvvvvv
-     			 */
-			if ((ch & 0xc0) != 0xc0) {
-				/* not a UTF-8 text */
-				return FALSE;
-			}
+	if (g_utf8_validate (sniff_buffer->buffer, 
+			     sniff_buffer->buffer_length, (const gchar**)&end))
+	{
+		return TRUE;
+	} else {
+		/* Check whether the string was truncated in the middle of
+		 * a valid UTF8 char, or if we really have an invalid
+		 * UTF8 string
+     		 */
+		gint remaining_bytes = sniff_buffer->buffer_length;
 
-			if ((ch & 0x20) == 0) {
-				/* check if this is a 2-byte UTF-8 letter */
-				++index;
-				if ((sniff_buffer->buffer[index] & 0xc0) != 0x80) {
-					return FALSE;
-				}
-			} else if ((ch & 0x30) == 0x20) {
-				/* check if this is a 3-byte UTF-8 letter */
-				if ((sniff_buffer->buffer[++index] & 0xc0) != 0x80
-				    || (sniff_buffer->buffer[++index] & 0xc0) != 0x80) {
-					return FALSE;
-				}
-			} else if ((ch & 0x38) == 0x30) {
-				/* check if this is a 4-byte UTF-8 letter */
-				if ((sniff_buffer->buffer[++index] & 0xc0) != 0x80
-				    || (sniff_buffer->buffer[++index] & 0xc0) != 0x80
-				    || (sniff_buffer->buffer[++index] & 0xc0) != 0x80) {
-					return FALSE;
-				}
-			}
-		}
-	}
+		remaining_bytes -= (end-((gchar*)sniff_buffer->buffer));
 	
-	return TRUE;
+ 		return (g_utf8_get_char_validated(end, remaining_bytes) == -2);
+	} 
 }
 
 static int bitrates[2][15] = {
@@ -830,13 +818,20 @@ enum {
 	GNOME_VFS_MP3_SNIFF_LENGTH = 256
 };
 
+/**
+ * _gnome_vfs_sniff_buffer_looks_like_mp3:
+ * @sniff_buffer: buffer to examine
+ *
+ * Return value: returns %TRUE if the contents of @sniff_buffer appear to
+ * be an MP3.
+ **/
 gboolean
-gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
+_gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
 {
 	unsigned long mp3_header;
 	int offset;
 	
-	if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_MP3_SNIFF_LENGTH) != GNOME_VFS_OK) {
+	if (_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_MP3_SNIFF_LENGTH) != GNOME_VFS_OK) {
 		return FALSE;
 	}
 
@@ -885,7 +880,7 @@ gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
 			 */
 			offset += 1 + length;
 
-			if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, offset + 4) != GNOME_VFS_OK) {
+			if (_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, offset + 4) != GNOME_VFS_OK) {
 				return FALSE;
 			}
 			mp3_header = get_4_byte_value (&sniff_buffer->buffer[offset]);
@@ -900,28 +895,3 @@ gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
 
 	return FALSE;
 }
-
-gboolean
-gnome_vfs_sniff_buffer_looks_like_gzip (GnomeVFSMimeSniffBuffer *sniff_buffer,
-	const char *file_name)
-{
-	if (sniff_buffer == NULL) {
-		return FALSE;
-	}
-	
-	if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 2) != GNOME_VFS_OK) {
-		return FALSE;
-	}
-	
-	if (sniff_buffer->buffer[0] != 0x1F || sniff_buffer->buffer[1] != 0x8B) {
-		/* not a gzipped file */
-		return FALSE;
-	}
-	
-	if (file_name == NULL) {
-		return TRUE;
-	}
-	
-	return TRUE;
-}
-
